module Examples where

import Data.Char (isUpper, toLower)
import StateMonad

import qualified SimpleRefs as SR
import Refs
import WriterMonad



-- examples for writer monad

ex1 :: Writer String Int
ex1 = do let a= 3+4
         tell $ "Calculated a " ++ show a ++ "\n"
	 let b= 8* 3
	 tell $ "Calculated b " ++ show b ++ "\n"
	 return $ a+ b

ex1a :: Logger Int
ex1a = do let a= 3+4
          logger $ "Calculated a: " ++ show a
	  let b= 8* 3
	  logger $ "Calculated b: " ++ show b
	  return $ a+ b


-- examples for state Monad
-- Example using a very simple state
type WithCounter alpha = State Int alpha

-- This function is like toLower, but also counts the number of upper-case
-- characters converted to lower.
cntToLower :: String-> (String, Int)
cntToLower s = run (cntToL s) 0

-- Main function : we can now use monad syntax!
cntToL :: String-> WithCounter String
cntToL [] = return ""
cntToL (x:xs)
  | isUpper x =  do ys<- cntToL xs 
                    set (+1)
                    return (toLower x: ys)
  | otherwise =  do { ys<- cntToL xs; return (x: ys) }


-- Examples for references

ex2 :: SR.Stateful Int String
ex2 = do r<- SR.newRef 43 
         s<- SR.newRef 17
	 SR.writeRef r 99
	 SR.writeRef s 17
	 v<- SR.readRef r
	 w<- SR.readRef s
	 return $ "r= "++ show v++ ", s= "++ show w

ex3 :: Stateful String
ex3 = do r<- newRef ""
         s<- newRef (0 :: Int)
	 writeRef r "Foo"
	 writeRef s 43
	 vr <- readRef r
         vs <- readRef s
	 return $ vr ++ show vs
