module Refs(
    Ref
  , Stateful
  , readRef
  , writeRef
  , newRef
  , runSt
  ) where

-- Typed references

import qualified Data.Map as Map
import Data.Map(Map)
import Text.Printf
import Data.Dynamic

import StateMonad

newtype Ref alpha  = Ref Int

-- The memory maps references to alpha  values, and keeps track of the next
--  fresh reference.
data Mem = Mem { fresh :: Int, mem   :: Map Int Dynamic }

-- A stateful computation is a state monad with a memory as a state
type Stateful beta = State Mem beta
   -- This is slightly unsafe because the type synonym is not opaque

-- Elementary operations

readRef :: Typeable alpha=> Ref alpha-> Stateful alpha
readRef (Ref i) = get $ \m->
  case Map.lookup i (mem m) of
    Just a -> case fromDynamic a of
                Just v -> v
                Nothing -> error $ printf "readRef: Refence %06x ill-typed" i
    Nothing -> error $ printf "readRef: Reference %06x not defined" i
writeRef :: Typeable alpha=> Ref alpha-> alpha-> Stateful ()
writeRef (Ref i) a  = set $ \m-> m{mem = Map.insert i (toDyn a) (mem m)}

newRef :: Typeable alpha=> alpha-> Stateful (Ref alpha)
newRef a  = do i <- get fresh
               set $ \m-> m {mem= Map.insert i (toDyn a) (mem m), fresh= i+1}
               return $ Ref i

-- Run alpha  stateful computation with an empty state
runSt :: Stateful beta -> beta
runSt s = fst $ StateMonad.run s $ Mem { fresh= 0, mem= Map.empty } 

