module SimpleRefs(
    Ref
  , Stateful
  , readRef
  , writeRef
  , newRef
  , runSt
  ) where

import qualified Data.Map as Map
import Data.Map(Map)
import Text.Printf

import StateMonad

newtype Ref = Ref Int

-- The memory maps references to a values, and keeps track of the next
--  fresh reference.
data Mem alpha = Mem { fresh :: Int, mem   :: Map Int alpha }

-- A stateful computation is a state monad with a memory as a state
type Stateful alpha beta = State (Mem alpha) beta

-- Elementary operations

readRef :: Ref-> Stateful alpha alpha
readRef (Ref i) = get $ \m->
  case Map.lookup i (mem m) of
    Just a -> a
    Nothing -> error $ printf "readRef: Reference %08x not defined." i

writeRef :: Ref-> alpha-> Stateful alpha ()
writeRef (Ref i) a = set $ \m-> m{mem = Map.insert i a (mem m)}

newRef :: alpha-> Stateful alpha Ref
newRef a = do i <- get fresh
              set $ \m-> m { mem= Map.insert i a (mem m), fresh= i+1 }
	      return $ Ref i

-- Run a stateful computation with a new state

runSt :: Stateful alpha beta -> beta
runSt s = fst $ StateMonad.run s $ Mem { fresh= 0, mem= Map.empty } 

