module WriterMonad where

import Data.Monoid

data Writer sigma alpha = W sigma alpha

instance Functor (Writer sigma) where
  fmap f (W s a) = W s $ f a

instance Monoid sigma=> Applicative (Writer sigma) where
  pure a = W mempty a
  W s1 f <*> W s2 a = W (s1 `mappend` s2) $ f a 

instance Monoid sigma=> Monad (Writer sigma) where
  return a = W mempty a
  (W s1 a) >>= g = let W s2 b = g a
                   in  W (s1 `mappend` s2) b

tell :: sigma-> Writer sigma ()
tell s = W s ()

runW :: Writer sigma a-> (sigma, a)
runW (W s a) = (s, a)

-- A logging monad

type Logger = Writer [String]

logger :: String-> Logger ()
logger s = tell [s] 

runLog :: Logger a-> IO a
runLog c = let (l, a) = runW c
           in  do putStrLn $ unlines l; return a
