import scala.util.Try

enum Operator:
  case Plus
  case Times
  case Minus
  case DivideBy

  override def toString: String = this match
    case Plus => "+"
    case Times => "*"
    case Minus => "-"
    case DivideBy => "/"

enum Expr:
  case Binary(operator: Operator, left: Expr, right: Expr)
  case Const(value: Int)
  case Variable(identifier: String)

  override def toString: String = this match
    case Binary(op,l,r) => s"$l $op $r"
    case Const(v) => v.toString
    case Variable(i) => i

object Expr:
  def eval(expr: Expr, env: Map[String,Int]): Option[Int] = 
    expr match 
      case Const(v) => Some(v)
      case Binary(op,l,r) => 
        val f: (Int,Int) => Int = op match 
          case Operator.Plus => _ + _
          case Operator.Minus => _ - _
          case Operator.Times => _ * _
          case Operator.DivideBy => _ / _
        for
          l <- eval(l,env)
          r <- eval(r,env)
          res <- Try(f(l,r)).recover{ case e if !e.isInstanceOf[ArithmeticException] => throw e }.toOption
        yield res
      case Variable(i) =>
        env.get(i)
